"use strict";
/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.DocumentLambda = void 0;
const server_services_core_1 = require("@fluidframework/server-services-core");
const server_services_telemetry_1 = require("@fluidframework/server-services-telemetry");
const contextManager_1 = require("./contextManager");
const documentPartition_1 = require("./documentPartition");
class DocumentLambda {
    constructor(factory, context, documentLambdaServerConfiguration) {
        this.factory = factory;
        this.context = context;
        this.documentLambdaServerConfiguration = documentLambdaServerConfiguration;
        this.documents = new Map();
        this.contextManager = new contextManager_1.DocumentContextManager(context);
        this.contextManager.on("error", (error, errorData) => {
            context.error(error, errorData);
        });
        this.activityCheckTimer = setInterval(this.inactivityCheck.bind(this), documentLambdaServerConfiguration.partitionActivityCheckInterval);
    }
    handler(message) {
        if (!this.contextManager.setHead(message)) {
            this.context.log?.warn("Unexpected head offset. " +
                `head offset: ${this.contextManager.getHeadOffset()}, message offset: ${message.offset}`);
            return undefined;
        }
        this.handlerCore(message);
        this.contextManager.setTail(message);
        return undefined;
    }
    close(closeType) {
        if (this.activityCheckTimer !== undefined) {
            clearInterval(this.activityCheckTimer);
            this.activityCheckTimer = undefined;
        }
        this.contextManager.close();
        for (const [, partition] of this.documents) {
            partition.close(closeType);
        }
        this.documents.clear();
    }
    handlerCore(message) {
        const boxcar = (0, server_services_core_1.extractBoxcar)(message);
        if (!boxcar.documentId || !boxcar.tenantId) {
            return;
        }
        // Stash the parsed value for down stream lambdas
        message.value = boxcar;
        // Create the routing key from tenantId + documentId
        const routingKey = `${boxcar.tenantId}/${boxcar.documentId}`;
        // Create or update the DocumentPartition
        let document = this.documents.get(routingKey);
        if (!document) {
            this.logMessageDetails(boxcar, message, boxcar.documentId, boxcar.tenantId);
            // Create a new context and begin tracking it
            const documentContext = this.contextManager.createContext(boxcar, message);
            document = new documentPartition_1.DocumentPartition(this.factory, boxcar.tenantId, boxcar.documentId, documentContext, this.documentLambdaServerConfiguration.partitionActivityTimeout);
            this.documents.set(routingKey, document);
        }
        else {
            // SetHead assumes it will always receive increasing offsets. So we need to split the creation case
            // from the update case.
            document.context.setHead(message);
        }
        // Forward the message to the document queue and then resolve the promise to begin processing more messages
        document.process(message);
    }
    logMessageDetails(boxcar, message, documentId, tenantId) {
        if (boxcar.contents?.length > 0) {
            const msgOffset = message.offset;
            const msgPartition = message.partition;
            const msgTopic = message.topic;
            const firstOp = boxcar.contents[0];
            let operationType = firstOp.type;
            let sequenceNumber = -1;
            let firstDocMsgType;
            let timestamp;
            const boxcarSize = boxcar.contents.length;
            switch (firstOp.type) {
                case server_services_core_1.RawOperationType: {
                    const rawOperationMessage = firstOp;
                    operationType = rawOperationMessage.type;
                    firstDocMsgType = rawOperationMessage.operation.type;
                    timestamp = rawOperationMessage.timestamp;
                    break;
                }
                case server_services_core_1.SequencedOperationType: {
                    const sequencedOperationMessage = firstOp;
                    operationType = sequencedOperationMessage.type;
                    firstDocMsgType = sequencedOperationMessage.operation.type;
                    sequenceNumber = sequencedOperationMessage.operation.sequenceNumber;
                    timestamp = sequencedOperationMessage.operation.timestamp;
                    break;
                }
                case server_services_core_1.NackOperationType: {
                    const nackMessage = firstOp;
                    operationType = nackMessage.type;
                    sequenceNumber = nackMessage.operation.sequenceNumber;
                    timestamp = nackMessage.timestamp;
                    break;
                }
                case server_services_core_1.SignalOperationType: {
                    const signalMessage = firstOp;
                    operationType = signalMessage.type;
                    timestamp = signalMessage.timestamp;
                }
                default:
                    // ignore unknown types
                    break;
            }
            const lumberjackProperties = {
                ...(0, server_services_telemetry_1.getLumberBaseProperties)(documentId, tenantId),
                msgOffset,
                msgPartition,
                msgTopic,
                boxcarSize,
                operationType,
                firstOpType: firstDocMsgType,
                sequenceNumber,
                timestamp,
            };
            server_services_telemetry_1.Lumberjack.info(`Creating new document partition`, lumberjackProperties);
        }
    }
    /**
     * Closes inactive documents
     */
    inactivityCheck() {
        const now = Date.now();
        const documentPartitions = Array.from(this.documents);
        for (const [routingKey, documentPartition] of documentPartitions) {
            if (documentPartition.isInactive(now)) {
                // Close and remove the inactive document
                this.contextManager.removeContext(documentPartition.context);
                documentPartition.close(server_services_core_1.LambdaCloseType.ActivityTimeout);
                this.documents.delete(routingKey);
            }
        }
    }
}
exports.DocumentLambda = DocumentLambda;
//# sourceMappingURL=documentLambda.js.map