"use strict";
/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.KafkaRunner = void 0;
const util_1 = require("util");
const serialize_error_1 = require("serialize-error");
const common_utils_1 = require("@fluidframework/common-utils");
const server_services_client_1 = require("@fluidframework/server-services-client");
const server_services_telemetry_1 = require("@fluidframework/server-services-telemetry");
const partitionManager_1 = require("./partitionManager");
class KafkaRunner {
    constructor(factory, consumer, config) {
        this.factory = factory;
        this.consumer = consumer;
        this.config = config;
        this.stopped = false;
        this.runnerMetric = server_services_telemetry_1.Lumberjack.newLumberMetric(server_services_telemetry_1.LumberEventName.KafkaRunner);
    }
    // eslint-disable-next-line @typescript-eslint/promise-function-async
    start(logger) {
        if (this.deferred) {
            this.runnerMetric.error("Runner already started");
            throw new Error("Already started");
        }
        this.deferred = new common_utils_1.Deferred();
        process.on("warning", (msg) => {
            console.trace("Warning", msg);
        });
        this.factory.on("error", (error) => {
            this.runnerMetric.error("Kafka factory encountered an error", error);
            this.deferred?.reject(error);
            this.deferred = undefined;
        });
        this.partitionManager = new partitionManager_1.PartitionManager(this.factory, this.consumer, logger, this.config);
        this.partitionManager.on("error", (error, errorData) => {
            const documentId = errorData?.documentId ?? "";
            const tenantId = errorData?.tenantId ?? "";
            const lumberProperties = (0, server_services_telemetry_1.getLumberBaseProperties)(documentId, tenantId);
            const metadata = {
                messageMetaData: {
                    documentId,
                    tenantId,
                },
            };
            this.runnerMetric.setProperties(lumberProperties);
            if (errorData && !errorData.restart) {
                const errorMsg = "KafkaRunner encountered an error that is not configured to trigger restart";
                logger?.error(errorMsg, metadata);
                logger?.error((0, util_1.inspect)(error), metadata);
                if (!this.runnerMetric.isCompleted()) {
                    this.runnerMetric.error(errorMsg, error);
                }
                else {
                    server_services_telemetry_1.Lumberjack.error(errorMsg, lumberProperties, error);
                }
            }
            else {
                const errorMsg = "KafkaRunner encountered an error that will trigger a restart";
                logger?.error(errorMsg, metadata);
                logger?.error((0, util_1.inspect)(error), metadata);
                if (!this.runnerMetric.isCompleted()) {
                    this.runnerMetric.error(errorMsg, error);
                }
                else {
                    server_services_telemetry_1.Lumberjack.error(errorMsg, lumberProperties, error);
                }
                this.deferred?.reject(error);
                this.deferred = undefined;
            }
        });
        this.stopped = false;
        return this.deferred.promise;
    }
    /**
     * Signals to stop the service
     */
    async stop(caller, uncaughtException) {
        if (this.stopped) {
            server_services_telemetry_1.Lumberjack.info("KafkaRunner.stop already called, returning early.");
            return;
        }
        this.stopped = true;
        server_services_telemetry_1.Lumberjack.info("KafkaRunner.stop starting.");
        try {
            // Stop listening for new updates
            await this.consumer.pause();
            // Stop the partition manager
            await this.partitionManager?.stop();
            // Dispose the factory
            await this.factory.dispose();
            // Close the underlying consumer, but setting a timeout for safety
            await (0, server_services_client_1.promiseTimeout)(30000, this.consumer.close());
            // Mark ourselves done once the partition manager has stopped
            if (caller === "uncaughtException") {
                this.deferred?.reject({
                    uncaughtException: (0, serialize_error_1.serializeError)(uncaughtException),
                }); // reject the promise so that the runService exits the process with exit(1)
            }
            else {
                this.deferred?.resolve();
            }
            this.deferred = undefined;
            if (!this.runnerMetric.isCompleted()) {
                this.runnerMetric.success("KafkaRunner stopped");
            }
            else {
                server_services_telemetry_1.Lumberjack.info("KafkaRunner stopped");
            }
        }
        catch (error) {
            if (!this.runnerMetric.isCompleted()) {
                this.runnerMetric.error("KafkaRunner encountered an error during stop", error);
            }
            else {
                server_services_telemetry_1.Lumberjack.error("KafkaRunner encountered an error during stop", undefined, error);
            }
            if (caller === "sigterm") {
                this.deferred?.resolve();
            }
            else {
                // uncaughtException
                this.deferred?.reject({
                    forceKill: true,
                    uncaughtException: (0, serialize_error_1.serializeError)(uncaughtException),
                    runnerStopException: (0, serialize_error_1.serializeError)(error),
                });
            }
            this.deferred = undefined;
            throw error;
        }
    }
}
exports.KafkaRunner = KafkaRunner;
//# sourceMappingURL=runner.js.map